// Ethernet Watchdog Board
// Relay Test
// Randall Hyde
//
// This example code is in the public domain.

#include <SPI.h>
#include <Ethernet.h>


// Pin 7 is the Heartbeat LED on the Ethernet WD board
// give it a name:

int led = (7);



#define eth_reset	(9)
#define wd_rfsh     (22)
#define wd_latch    (21)
#define Reset       (23)


// Eight output pins connected to ULN2803 Darlington array.
// Note: Pins are reversed to improve the PCB layout.
// Relay outputs:

#define out0 (24)
#define out1 (25)
#define out2 (26)
#define out3 (27)
#define out4 (28)
#define out5 (29)
#define out6 (30)
#define out7 (31)




// Turn off all the digital outputs:

void clearAll( void )
{
    	digitalWrite( out0, 0 );
		digitalWrite( out1, 0 );
		digitalWrite( out2, 0 );
		digitalWrite( out3, 0 );
		digitalWrite( out4, 0 );
		digitalWrite( out5, 0 );
		digitalWrite( out6, 0 );
		digitalWrite( out7, 0 );
}


// Must send a positive pulse to the TA8030s
// to refresh the watchdog timer:

void watchdogRefresh( void )
{
    digitalWrite( wd_rfsh, 1 );
    delay( 1 );
    digitalWrite( wd_rfsh, 0 );
}

// Must send a negative-going pulse to the TA8030S
// chip to cause a reset. The pulse must be at least
// 350 msec wide.

void toggleReset( void )
{
        digitalWrite( Reset, 1 );
        delay( 500 );
        digitalWrite( Reset, 0 );
        delay( 500 );
        digitalWrite( Reset, 1 );
}



// the setup routine runs once when you press reset:

void setup()
{
	byte latch;

    // Initialize the Ethernet featherwing:

	Serial.begin( 9600 );
	while(!Serial);	// Wait for serial port to come up.
	Serial.println( "Plantation Productions' EWB relay test" );


    // Initialize the LED digital pin as an output.

    pinMode(led, OUTPUT);

    // Digital output lines -- always outputs:

    pinMode( out0, OUTPUT );
    pinMode( out1, OUTPUT );
    pinMode( out2, OUTPUT );
    pinMode( out3, OUTPUT );
    pinMode( out4, OUTPUT );
    pinMode( out5, OUTPUT );
    pinMode( out6, OUTPUT );
    pinMode( out7, OUTPUT );


	clearAll();					// Assume time-out (failsafe) on startup.

    // Stuff needed by the watchdog timer:

    pinMode( eth_reset, OUTPUT );   // Ethernet reset pin is an output
    pinMode( Reset, OUTPUT );       // Reset pin is an output
    pinMode( wd_rfsh, OUTPUT );     // Watchdog refresh is an output
    pinMode( wd_latch, INPUT );     // Watchdog Latch is an input


    // Loop until we've reset the watchdog timer and read a "watchdog latch
    // not set" signal from the watchdog timer.

    do
    {
		toggleReset();                  // Reset watchdog timer latch.
		watchdogRefresh();              // Must do this at least once every 4.7 seconds
		latch = digitalRead( wd_latch );
		#if useSerialPort

			if( !latch )
			{
				Serial.println( "Watchdog latch not cleared, resetting" );
			}

		#endif

	}while( !latch );

}

// the loop routine runs over and over again forever:

void loop()
{
	static unsigned long	lastSecs	= 0;
	static unsigned long	seconds		= 0;
	static unsigned long 	lastTime 	= 0;
	static byte 			ledState	= 0;
	static int				relayState	= 0;
	static int				relayVal[8]	= {0, 0, 0, 0, 0, 0, 0, 0};

			unsigned long 	curTime 	= millis();


	// Every 1/2 second invert the LED and
	// refresh the watchdog timer.

	if( abs( curTime - lastTime ) >= 1000 )
	{

    	digitalWrite(led, ledState);
    	ledState = !ledState;
    	watchdogRefresh();
    	lastTime = curTime;

    	// If the Weensy watchdog timer has timed out,
    	// reset it (because we're obviously running).

    	byte latch = digitalRead( wd_latch );
    	if( !latch )
    	{
    		#if useSerialPort

				Serial.println( "Watchdog latch clear, resetting" );

			#endif

			setup();	// Reset everything
		}
		++seconds;
	}

	// Every two seconds, change the relay state:

	if( seconds != lastSecs )
	{
		Serial.print( "Relay State=" ); Serial.println( relayState );
		memset( relayVal, 0, sizeof relayVal );
		switch( relayState )
		{
			case 0:
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				relayVal[ relayState ] = 1;
				break;

			case 8:
				relayVal[0] = 1;
				relayVal[1] = 1;
				relayVal[2] = 1;
				relayVal[3] = 1;
				break;

			case 9:
				relayVal[4] = 1;
				relayVal[5] = 1;
				relayVal[6] = 1;
				relayVal[7] = 1;
				break;

			case 10:
				// All Relays Off
				break;

			case 11:
				relayVal[0] = 1;
				relayVal[1] = 1;
				relayVal[2] = 1;
				relayVal[3] = 1;
				relayVal[4] = 1;
				relayVal[5] = 1;
				relayVal[6] = 1;
				relayVal[7] = 1;
				break;

			case 12:
				// All Relays Off
				break;
		}
		digitalWrite( out0, relayVal[0] );
		digitalWrite( out1, relayVal[1] );
		digitalWrite( out2, relayVal[2] );
		digitalWrite( out3, relayVal[3] );
		digitalWrite( out4, relayVal[4] );
		digitalWrite( out5, relayVal[5] );
		digitalWrite( out6, relayVal[6] );
		digitalWrite( out7, relayVal[7] );

		lastSecs = seconds;
		relayState = (relayState + 1) % 13;
	}
}
